package sisc.modules.hashtable;

import java.io.IOException;

import sisc.io.ValueWriter;
import sisc.ser.Serializer;
import sisc.ser.Deserializer;
import sisc.util.ExpressionVisitor;
import sisc.data.Value;
import sisc.data.Pair;
import sisc.data.Procedure;

public abstract class HashtableBase extends Value {

    public abstract Procedure getEqualsProc();
    public abstract Procedure getHashProc();
    public abstract Value get(Value k);
    public abstract Value put(Value k, Value v);
    public abstract Value remove(Value k);
    public abstract int size();
    public abstract void clear();
    public abstract void addAList(Pair p);
    public abstract Pair toAList();
    public abstract Pair keys();
    public abstract boolean valueEqual(Value v);
    public abstract int valueHashCode();
    public abstract void serialize(Serializer s) throws IOException;
    public abstract void deserialize(Deserializer s) throws IOException;
    public abstract boolean visit(ExpressionVisitor v);
    public abstract void display(ValueWriter w) throws IOException;
}

/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is the Second Interpreter of Scheme Code (SISC).
 * 
 * The Initial Developer of the Original Code is Scott G. Miller.
 * Portions created by Scott G. Miller are Copyright (C) 2000-2007
 * Scott G. Miller.  All Rights Reserved.
 * 
 * Contributor(s):
 * Matthias Radestock 
 * 
 * Alternatively, the contents of this file may be used under the
 * terms of the GNU General Public License Version 2 or later (the
 * "GPL"), in which case the provisions of the GPL are applicable 
 * instead of those above.  If you wish to allow use of your 
 * version of this file only under the terms of the GPL and not to
 * allow others to use your version of this file under the MPL,
 * indicate your decision by deleting the provisions above and
 * replace them with the notice and other provisions required by
 * the GPL.  If you do not delete the provisions above, a recipient
 * may use your version of this file under either the MPL or the
 * GPL.
 */
