package sisc.ser;

import java.io.*;

public class MemoryRandomAccessInputStream extends SeekableInputStream {
    
    protected byte[] data;
    protected int filePointer;

    public MemoryRandomAccessInputStream(InputStream in) throws IOException {
        ByteArrayOutputStream bos=new ByteArrayOutputStream();
        int rc=0;
        byte[] buffer=new byte[65536];
        while (-1!=(rc=in.read(buffer))) {
            if (rc>0)
                bos.write(buffer, 0, rc);
        }
        bos.flush();
        data=bos.toByteArray();
    }

    public void seek(long pos) throws IOException {
        filePointer=(int)pos;
    }

    public long getFilePointer() throws IOException {
        return filePointer;
    }

    public int read() throws IOException {
        if (filePointer==data.length) return -1;
        return data[filePointer++] & 0xff;
    }

    public int read(byte[] b, int off, int len) throws IOException {
        int rc=Math.min(len, data.length - filePointer);
        if (rc==0 && len>0) return -1;
        System.arraycopy(data, filePointer, b, off, rc);
        filePointer+=rc;
        return rc;
    }

    public int skipBytes(int bc) throws IOException {
        int rc=Math.min(bc, data.length - filePointer);
        filePointer+=rc;
        return rc;
    }
}

/*
 * The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is the Second Interpreter of Scheme Code (SISC).
 * 
 * The Initial Developer of the Original Code is Scott G. Miller.
 * Portions created by Scott G. Miller are Copyright (C) 2000-2007
 * Scott G. Miller.  All Rights Reserved.
 * 
 * Contributor(s):
 * Matthias Radestock 
 * 
 * Alternatively, the contents of this file may be used under the
 * terms of the GNU General Public License Version 2 or later (the
 * "GPL"), in which case the provisions of the GPL are applicable 
 * instead of those above.  If you wish to allow use of your 
 * version of this file only under the terms of the GPL and not to
 * allow others to use your version of this file under the MPL,
 * indicate your decision by deleting the provisions above and
 * replace them with the notice and other provisions required by
 * the GPL.  If you do not delete the provisions above, a recipient
 * may use your version of this file under either the MPL or the
 * GPL.
 */
